@@ -1,72 +1,77 @@
 get_type <- function(fmt) {
     type_list <- list(
-        txt = "tsv",
+        clipboard = "clipboard",
+        # supported formats
+        "," = "csv",
+        ";" = "csv2",
         "\t" = "tsv",
-        tsv = "tsv",
+        "|" = "psv",
+        arff = "arff",
         csv = "csv",
         csv2 = "csv2",
         csvy = "csvy",
-        "," = "csv",
-        ";" = "csv2",
-        psv = "psv",
-        "|" = "psv",
-        fwf = "fwf",
-        rds = "rds",
-        rdata = "rdata",
-        dta = "dta",
-        stata = "dta",
         dbf = "dbf",
         dif = "dif",
-        spss = "sav",
-        sav = "sav",
+        dta = "dta",
+        dump = "dump",
+        epiinfo = "rec",
+        excel = "xlsx",
+        feather = "feather",
+        fortran = "fortran",
+        fst = "fst",
+        fwf = "fwf",
+        htm = "html",
+        html = "html",
+        json = "json",
+        mat = "matlab",
+        matlab = "matlab",
+        minitab = "mtp",
+        mtp = "mtp",
+        ods = "ods",
+        psv = "psv",
+        r = "r",
+        rda = "rdata",
+        rdata = "rdata",
+        rds = "rds",
+        rec = "rec",
         sas = "sas7bdat",
         sas7bdat = "sas7bdat",
-        xpt = "xpt",
-        xport = "xpt",
-        mtp = "mtp",
-        minitab = "mtp",
+        sav = "sav",
+        spss = "sav",
+        stata = "dta",
         syd = "syd",
         systat = "syd",
-        json = "json",
-        rec = "rec",
-        epiinfo = "rec",
-        arff = "arff",
+        tsv = "tsv",
+        txt = "tsv",
         weka = "arff",
         xls = "xls",
         xlsx = "xlsx",
-        excel = "xlsx",
-        fortran = "fortran",
-        dump = "dump",
-        r = "r",
-        clipboard = "clipboard",
-        ods = "ods",
-        htm = "html",
-        html = "html",
         xml = "xml",
-        yml = "yml",
+        xport = "xpt",
+        xpt = "xpt",
         yaml = "yml",
+        yml = "yml",
         # compressed formats
-        zip = "zip",
-        tar = "tar",
-        gz = "gzip",
-        gzip = "gzip",
         csv.gz = "gzip",
         csv.gzip = "gzip",
+        gz = "gzip",
+        gzip = "gzip",
+        tar = "tar",
+        zip = "zip",
         # known but unsupported formats
+        bmp = "bmp",
+        gexf = "gexf",
         gnumeric = "gnumeric",
         jpeg = "jpg",
         jpg = "jpg",
-        png = "png",
-        bmp = "bmp",
-        tif = "tiff",
-        tiff = "tiff",
-        sss = "sss",
-        sdmx = "sdmx",
         mat = "matlab",
         matlab = "matlab",
-        gexf = "gexf",
         npy = "npy",
-        feather = 'feather'
+        png = "png",
+        sdmx = "sdmx",
+        sss = "sss",
+        tif = "tiff",
+        tiff = "tiff"
     )
     out <- type_list[[tolower(fmt)]]
     if (is.null(out)) {
@@ -83,7 +88,8 @@ get_ext <- function(file) {
     if (!grepl("^http.*://", file)) {
         fmt <- file_ext(file)
     } else if(grepl("^http.*://", file)) {
-        file <- url_parse(file)$path
+        parsed <- strsplit(strsplit(file, "?", fixed = TRUE)[[1]][1], "/", fixed = TRUE)[[1]]
+        file <- parsed[length(parsed)]
         fmt <- file_ext(file)
         get_type(fmt)
     }
