@@ -1,10 +1,6 @@
 context("Excel imports/exports")
 require("datasets")
 
-#test_that("Export to Excel (.xls)", {})
-#test_that("Import from Excel (.xls)", {})
-#unlink("iris.xls")
-
 test_that("Export to Excel (.xlsx)", {
     expect_true(export(iris, "iris.xlsx") %in% dir())
 })
@@ -12,6 +8,8 @@ test_that("Export to Excel (.xlsx)", {
 test_that("Import from Excel (.xlsx)", {
     expect_true(is.data.frame(import("iris.xlsx", readxl = FALSE)))
     expect_true(is.data.frame(import("iris.xlsx", readxl = TRUE)))
+    expect_true(is.data.frame(import("iris.xlsx", sheet = 1)))
+    expect_true(is.data.frame(import("iris.xlsx", which = 1)))
 })
 
 unlink("iris.xlsx")
